package org.simplericity.datadirlocator;

/*
 * Copyright 2007 Eirik Bjorsnos.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


import org.apache.log4j.Logger;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletContext;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;
import java.util.Collections;
import java.util.ArrayList;
import java.util.List;

/**
 * Will locate a data directory by:
 * <ol>
 * <li>Checking for the existence of a JNDI entry</li>
 * <li>Checking for the existence of a context parameter</li>
 * <li>Checking for the existence of a system property</li>
 * <li>Checking for the existence of an environment variable</li>
 * </ol>
 * If all these checks fail, a default dataDir is used.
 */
public class DefaultDataDirectoryLocator implements DataDirectoryLocator {

    private Logger log = Logger.getLogger(getClass());
    private String jndiName = "java:comp/env/dataDirectory";
    private String systemProperty ="dataDirectory";
    private String defaultDir = "$HOME/datadirectory";
    private String envVarName = "DATADIRECTORY";

    private ServletContext servletContext;

    private String contextParamName = "dataDirectory";
    private List<DataDirectoryLocatorStrategy> strategies = Collections.<DataDirectoryLocatorStrategy>synchronizedList(new ArrayList<DataDirectoryLocatorStrategy>());
    private DataDirectoryLocatorStrategy defaultStrategy;

    public DefaultDataDirectoryLocator() {
        strategies.add(new JndiStrategy());
        strategies.add(new ContextParamStrategy());
        strategies.add(new SystemPropertyStrategy());
        strategies.add(new EnvironmentVariableStrategy());
        defaultStrategy = new DefaultStrategy();
    }

    /**
     * Locate the data directory.
     *
     * @return A File pointing to the data directory
     */
    public File locateDataDirectory() {

        for(DataDirectoryLocatorStrategy strategy : strategies) {
            File dir = strategy.locateDataDirectory();
            if(dir != null) {
                return dir;
            }
        }
        return defaultStrategy.locateDataDirectory();

    }

    private String interpolateDirectory(String defaultDir) {
        String home = System.getProperty("user.home").replace(File.separatorChar, '/');
        return defaultDir == null ? null : defaultDir.replaceAll("\\$HOME", home);
    }

    /**
     * Sets the JNDI name that will be looked for.
     * Example: <code>java:comp/env/application/dataDir</code>
     *
     * @param jndiName The jndi name to look for.
     */
    public void setJndiName(String jndiName) {
        this.jndiName = jndiName;
    }

    /**
     * Sets the system property that will be looked for.
     * Example: <code>application.dataDir</code>
     *
     * @param systemProperty the system propery to check for.
     */
    public void setSystemProperty(String systemProperty) {
        this.systemProperty = systemProperty;
    }


    /**
     * Sets the default data directory that will be used if all other checks fail.
     * The string <code>$HOME</code> will be replaced by the user's home directory as given by System.getProperty("user.home").
     * Example: <code>$HOME/.applicationdata</code>
     *
     * @param defaultDir The default directory to fall back to.
     */
    public void setDefaultDir(String defaultDir) {
        this.defaultDir = defaultDir;
    }

    /**
     * Sets the servlet context to check for a context parameter in. If this is not set the context param
     * check will not be performed. Il this is set, {@link #setContextParamName(String)} must also be set.
     *
     * @param servletContext
     */
    public void setServletContext(ServletContext servletContext) {
        this.servletContext = servletContext;
    }

    /**
     * Sets the name of the context parameter to look for.
     *
     * @param contextParamName
     */
    public void setContextParamName(String contextParamName) {
        this.contextParamName = contextParamName;
    }

    public void addStrategy(DataDirectoryLocatorStrategy strategy) {
        strategies.add(strategy);
    }

    public void setEnvVarName(String envVarName) {
        this.envVarName = envVarName;
    }


    class JndiStrategy implements DataDirectoryLocatorStrategy {
        public File locateDataDirectory() {
            // 1. JNDI env-entry
            String jndiString = null;

            if (jndiName != null) {
                Object jndiObject = null;
                InitialContext context = null;

                try {
                    context = new InitialContext();

                    jndiObject = context.lookup(jndiName);
                } catch (NamingException e) {
                    log.info("Name " +jndiName +" is not bound");
                } finally {
                    if (context != null) {
                        try {
                            context.close();
                        } catch (NamingException e) {
                            log.error(e.getMessage(), e);
                        }
                    }
                }


                if (jndiObject != null && !(jndiObject instanceof String)) {
                    log.error("The env-entry \"" + jndiName + "\" is of type " + jndiObject.getClass() + " not " + String.class + " as expected");
                } else {
                    jndiString = (String) jndiObject;
                }

            }

            return jndiString == null ? null : new File(jndiString);
        }
    }

    class ContextParamStrategy implements DataDirectoryLocatorStrategy {
        public File locateDataDirectory() {
            // 2. Context parameter
            if (servletContext != null && contextParamName != null) {
                final String paramValue = servletContext.getInitParameter(contextParamName);
                return paramValue == null ? null : new File(paramValue);
            } else {
                return null;
            }
        }
    }

    class SystemPropertyStrategy implements DataDirectoryLocatorStrategy {
        public File locateDataDirectory() {
            // 3. System property
            if (systemProperty != null) {
                final String propValue = System.getProperty(systemProperty);
                return propValue == null ? null : new File(propValue);
            } else {
                return null;
            }
        }

    }

    class EnvironmentVariableStrategy implements DataDirectoryLocatorStrategy {
        public File locateDataDirectory() {
            final String value = getEnvironmentVariable(envVarName);
            return value == null ? null : new File(value);
        }
    }

    protected String getEnvironmentVariable(String envVarName) {
        return System.getenv(envVarName);
    }

    class DefaultStrategy implements DataDirectoryLocatorStrategy {

        public File locateDataDirectory() {

            if (defaultDir == null) {
                throw new IllegalStateException("defaultDir is not set and all other checks failed");
            }

            log.info("dataDir not configured, falling back to " + defaultDir);
            String dataDir = defaultDir;

            dataDir = interpolateDirectory(dataDir);
            dataDir = dataDir.replace('/', File.separatorChar);
            return new File(dataDir);
        }

    }
}
