package org.simplericity.datadirlocator.spring;

/*
 * Copyright 2007 Eirik Bjorsnos.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.simplericity.datadirlocator.DataDirectoryLocator;
import org.simplericity.datadirlocator.DefaultDataDirectoryLocator;
import org.apache.log4j.Logger;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.config.BeanFactoryPostProcessor;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.beans.factory.config.PropertyPlaceholderConfigurer;
import org.springframework.web.context.ServletContextAware;
import org.springframework.core.PriorityOrdered;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ApplicationContext;

import javax.servlet.ServletContext;
import java.io.File;
import java.util.Properties;

/**
 * BeanFactoryPostProcessor implementation that will replace Spring context references to <code>${dataDir}</code>
 * with the data directory returned by the provided {@link org.simplericity.datadirlocator.DataDirectoryLocator}.
 * The dataDirectory File object will also be exposed as a ServletContext attribute with key {@link #SERVLET_CONTEXT_ATTR}
 */
public class DataDirectoryPropertyReplacer implements BeanFactoryPostProcessor, InitializingBean, ServletContextAware, PriorityOrdered {
    private Logger log = Logger.getLogger(getClass());

    private DataDirectoryLocator dataDirectoryLocator;

    private boolean create = true;

    private String dataDirProperty = "dataDir";
    private String dataDirBeanName = "dataDir";
    private ServletContext servletContext;
    private int order = Ordered.HIGHEST_PRECEDENCE;
    private String defaultDirectory;

    public static final String SERVLET_CONTEXT_ATTR = DataDirectoryPropertyReplacer.class.getName() +"_contextattr";


    public void postProcessBeanFactory(ConfigurableListableBeanFactory configurableListableBeanFactory) throws BeansException {

        PropertyPlaceholderConfigurer configurer = new PropertyPlaceholderConfigurer();

        log.info("Determining data directory where configuration and persistent files will be stored");


        File dataDir = dataDirectoryLocator.locateDataDirectory();

        if(!dataDir.exists() && create) {
            log.info("Data directory " + dataDir.getAbsolutePath() + " does not exist, so we'll create it");
            dataDir.mkdirs();
        }

        Properties p = new Properties();
        p.setProperty(dataDirProperty, dataDir.getAbsolutePath());

        configurer.setProperties(p);
        configurer.setIgnoreUnresolvablePlaceholders(true);
        configurer.postProcessBeanFactory(configurableListableBeanFactory);


        servletContext.setAttribute(SERVLET_CONTEXT_ATTR, dataDir);

        if(dataDirBeanName != null) {
            configurableListableBeanFactory.registerSingleton(dataDirBeanName, dataDir);
        }

    }

    public void setDataDirectoryLocator(DataDirectoryLocator dataDirectoryLocator) {
        this.dataDirectoryLocator = dataDirectoryLocator;
    }

    public void setCreate(boolean create) {
        this.create = create;
    }

    public void setDataDirProperty(String dataDirProperty) {
        this.dataDirProperty = dataDirProperty;
    }

    public void setServletContext(ServletContext servletContext) {
        this.servletContext = servletContext;
    }

    public void afterPropertiesSet() throws Exception {
        // Set up the default data directory locator if none is set
        if(this.dataDirectoryLocator  == null) {
            this.dataDirectoryLocator = createDefaultDataDirectoryLocator();
        }
    }

    private DataDirectoryLocator createDefaultDataDirectoryLocator() {
        DefaultDataDirectoryLocator locator = new DefaultDataDirectoryLocator();
        locator.setServletContext(this.servletContext);
        if(defaultDirectory != null) {
            locator.setDefaultDir(defaultDirectory);
        }
        return locator;
    }

    public int getOrder() {
        return this.order;
    }

    public void setOrder(int order) {
        this.order = order;
    }

    public void setDefaultDirectory(String defaultDirectory) {
        this.defaultDirectory = defaultDirectory;
    }


    public void setDataDirBeanName(String dataDirBeanName) {
        this.dataDirBeanName = dataDirBeanName;
    }
}
